    SUBROUTINE KFDRIVE(HTOP,HBOT,CNVTOP,CNVBOT)
!     ******************************************************************
!$$$  SUBPROGRAM DOCUMENTATION BLOCK
!                .      .    .
! SUBPROGRAM:    KFDRIVE     CONVECTIVE PRECIPITATION PARAMETERIZATION
!   PRGRMMR: KAIN            ORG: W/NP2      DATE: 00-03-30

! ABSTRACT:
!     KFDRIVE IS THE MAIN DRIVER FOR THE KAIN-FRITSCH CONVECTION.
!     GRID POINTS ARE ISOLATED TO BE CHECKED FOR POSSIBLE
!     INITAITION OF THE K-F SCHEME.


! PROGRAM HISTORY LOG:
!   ??-??-??  KAIN       - ORIGINATOR
!   00-04-13  BLACK      - INCORPORATED INTO ETA MODEL

! USAGE: CALL KFDRIVE FROM MAIN PROGRAM EBU

!   INPUT ARGUMENT LIST:
!     NONE

!   OUTPUT ARGUMENT LIST:
!     HTOP   - CONVECTIVE CLOUD TOP (MODEL LAYER) FOR RADTN
!     HBOT   - CONVECTIVE CLOUD BOTTOM (MODEL LAYER) FOR RADTN
!     CNVTOP - CONVECTIVE CLOUD TOP (MODEL LAYER) FOR CHKOUT
!     CNVBOT - CONVECTIVE CLOUD BOTTOM (MODEL LAYER) FOR CHKOUT

!   SUBPROGRAMS CALLED:

!     UNIQUE:
!        KFPARA

!     LIBRARY:
!        NONE

!   COMMON BLOCKS: CTLBLK
!                  VRBLS
!                  PVRBLS
!                  MASKS
!                  CONTIN
!                  DYNAM
!                  LOOPS
!                  KFFDBK

! ATTRIBUTES:
!   LANGUAGE: FORTRAN 90
!   MACHINE : IBM SP
!$$$
!----------------------------------------------------------------------
    INCLUDE "parmeta.f90"
    INCLUDE "mpp.h"
    INCLUDE "mpif.h"
!----------------------------------------------------------------------
    PARAMETER(LP1=LM+1,JAM=6+2*(JM-10))
!----------------------------------------------------------------------
    INCLUDE "COMM_CTLBLK.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_VRBLS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_PVRBLS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_MASKS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_CONTIN.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_DYNAM.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_LOOPS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_KFFDBK.f90"
!----------------------------------------------------------------------
    PARAMETER(CP=1004.6,XLV=2.5E6,G=9.8)
    PARAMETER(ALIQ=613.3,BLIQ=17.502,CLIQ=4780.8 &
    ,         DLIQ=32.19)
!----------------------------------------------------------------------
    REAL &
    HTOP  (IDIM1:IDIM2,JDIM1:JDIM2),HBOT  (IDIM1:IDIM2,JDIM1:JDIM2) &
    ,CNVTOP(IDIM1:IDIM2,JDIM1:JDIM2),CNVBOT(IDIM1:IDIM2,JDIM1:JDIM2)

    REAL &
    SEM(LM),SEMS(LM),PRS(LM),TV0(LM),Z00(LM),SED(LM),DP(LM)

    INTEGER &
    ICUYES(IDIM1:IDIM2),LSB(IDIM1:IDIM2),NSHALL(JDIM1:JDIM2)
!------------------------------------------------------------------
    ROVG=R/G
!------------------------------------------------------------------
!...NOTE:  SEARCHING ONLY LOWEST 200 mb!!!!!
!------------------------------------------------------------------
    DO L=1,LM
        DO J=MYJS,MYJE
            DO I=MYIS,MYIE
                W0=-OMGALF(I,J,L)*CP/(G*DT)
                W0AVG(I,J,L)=(W0AVG(I,J,L)*(TST-1.)+W0)/TST
            ENDDO
        ENDDO
    ENDDO

! LG      IF(MOD(NTSD-NCLDCK/2,NCLDCK).NE.0)RETURN
    IF(MOD(NTSD,NCLDCK) /= 0)RETURN
    if (mype == 0) write(*,*)"CALL KF"

    DO J=JDIM1,JDIM2
        NSHALL(J)=0
    ENDDO
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

!***  MAKE A QUICK FOR CHECK FOR THESE THINGS THAT CAN ELIMINATE
!***  GRID POINTS FROM THE POSSIBILITY OF CONVECTIVE INITIATION:

!        1.) CONVECTION ALREADY ACTIVE AT THIS POINT
!        2.) POINT CLOSE TO GRID-DOMAIN BOUNDARY
!        3.) DOWNWARD MOTION AT ALL LEVELS IN LOWEST 300 MB
!        4.) NO CAPE
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

! omp parallel do
! omp& private(dp,es,i,icuyes,isink,l,l200,l400,lg,lsb,
! omp&         ncuyes,p200,p400,prs,psfck,qss,qtmp,
! omp&         sed,sem,sems,ttmp,tv0,z00)
    DO 100 J=MYJS2,MYJE2
    
        NCUYES=0
    
        DO I=MYIS,MYIE
            ICUYES(I)=0
            LSB(I)=0
        ENDDO
    
        DO 60 I=MYIS1,MYIE1
        
            IF(NCA(I,J) > NCLDCK .OR. HBM2(I,J) == 0.)GO TO 60
            LG=LMH(I,J)
            PSFCK=PD(I,J)*RES(I,J)*AETA(LG)+PT
        !     P300=PSFCK-3.E4
            P200=PSFCK-2.E4
            P400=PSFCK-4.E4
        
            DO L=LG,1,-1
                PRS(L)=PD(I,J)*RES(I,J)*AETA(L)+PT
            ENDDO
        
            DO L=LG,1,-1
                IF(PRS(L) > P200)L200=L
                IF(PRS(L) > P400)L400=L
            ENDDO
        
        !***  VERTICAL VELOCITY MUST BE UPWARD AT SOME LEVEL IN THE
        !***  LOWEST 300 MB...
        !***  LOWEST 400 MB...!!!!! 9/25/97 JSK
        
            ISINK=0
        
        !     DO L=LG,L400,-1
            DO L=LG,L200,-1
            !       IF(OMGALF(I,J,L).LT.0.)GO TO 25
                IF(OMGALF(I,J,L) < 0. .OR. Q2(I,J,L) > 1.)GO TO 25
            ENDDO
        
            ISINK=1
        
            25 CONTINUE
        
        !***  CALCULATE MOIST STATIC ENERGY AND SATURATION MOIST STATIC ENERGY
        !***  AT EACH VERTICAL LEVEL.
        
            DO L=LG,1,-1
                TTMP=T(I,J,L)
                QTMP=Q(I,J,L)
                DP(L)=(ETA(L+1)-ETA(L))*PD(I,J)*RES(I,J)
                TV0(L)=TTMP*(1.+0.608*QTMP)
            
                IF(L == LG)THEN
                    Z00(L)=0.
                ELSE
                    Z00(L)=Z00(L+1)-ROVG*0.5*(TV0(L)+TV0(L+1))* &
                    ALOG(PRS(L)/PRS(L+1))
                ENDIF
            
                ES=ALIQ*EXP((BLIQ*TTMP-CLIQ)/(TTMP-DLIQ))
                QSS=0.622*ES/(PRS(L)-ES)
                SED(L)=CP*TTMP+G*Z00(L)
                SEMS(L)=SED(L)+XLV*QSS
                SEM(L)=SED(L)+XLV*QTMP
            ENDDO
        
        !***  IF AIR IS SINKING EVERYWHERE IN LOWEST 400 MB,
        !***  REQUIRE SUPERADIABATIC LAYER IN LOWEST 200 MB
        !***  BEFORE CHECKING FOR CONVECTION IN KF SCHEME.
        
            IF(ISINK == 1)THEN
                DO L=LG,L200-1,-1
                    DO NL=L-1,L200,-1
                        IF(SED(L) > SED(NL))THEN
                            NCUYES=NCUYES+1
                            ICUYES(NCUYES)=I
                            LSB(I)=LG-L+1
                            GO TO 60
                        ENDIF
                    ENDDO
                ENDDO
            ELSE
            
            !***  IF THERE IS UPWARD MOTION, REQUIRE CONDITIONAL INSTABILITY
            !***  FOR A PARCEL ORIGINATING IN THE LOWEST 400 MB.
            
            !       DO L=LG,L400,-1
                DO L=LG,L200,-1
                    DO NL=L-1,1,-1
                        IF(SEM(L) > SEMS(NL))THEN
                            NCUYES=NCUYES+1
                            ICUYES(NCUYES)=I
                            LSB(I)=LG-L+1
                            GO TO 60
                        ENDIF
                    ENDDO
                ENDDO
            
            ENDIF
        !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            CONTINUE       ! End of I loop
        60 END DO
    !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        IF(NCUYES > 0)THEN
            CALL KFPARA(NCUYES,ICUYES,J,LSB,HTOP,HBOT &
            ,             CNVTOP,CNVBOT,NSHALL(J))
        ENDIF
    !-----------------------------------------------------------------
        CONTINUE       ! End of J loop
    100 END DO
!-----------------------------------------------------------------
    NSHALLOW=0

    DO J=MYJS2,MYJE2
        NSHALLOW=NSHALLOW+NSHALL(J)
    ENDDO

    CALL MPI_REDUCE(NSHALLOW,NSHAL,1,MPI_INTEGER,MPI_SUM,0 &
    ,               MPI_COMM_COMP,IRECV)
!-----------------------------------------------------------------
!     IF(MYPE.EQ.0)THEN
!       PRINT*,'at NTSD =',NTSD,' NSHALL =',NSHAL
!     ENDIF
!-----------------------------------------------------------------
    RETURN
    END SUBROUTINE KFDRIVE
