    SUBROUTINE KFTEND
!     ******************************************************************
!$$$  SUBPROGRAM DOCUMENTATION BLOCK
!                .      .    .
! SUBPROGRAM:    KFDRIVE     CONVECTIVE PRECIPITATION PARAMETERIZATION
!   PRGRMMR: KAIN            ORG: W/NP2      DATE: 00-03-30

! ABSTRACT:
!     KFTEND FEEDS TENDENCIES GENERATED BY THE KAIN-FRITSCH CONVECTIVE
!     SCHEME INTO THE GRIDSCALE TEMPERATURE AND MOISTURE FIELDS


! PROGRAM HISTORY LOG:
!   ??-??-??  KAIN       - ORIGINATOR
!   00-04-17  BLACK      - INCORPORATED INTO ETA MODEL

! USAGE: CALL KFTEND FROM MAIN PROGRAM EBU

!   INPUT ARGUMENT LIST:
!     NONE

!   OUTPUT ARGUMENT LIST:
!     NONE

!   SUBPROGRAMS CALLED:

!     UNIQUE:
!        KFPARA

!     LIBRARY:
!        NONE

!   COMMON BLOCKS: CTLBLK
!                  VRBLS
!                  DYNAM
!                  PVRBLS
!                  ACMCLH
!                  ACMPRE
!                  CNVCLD
!                  MASKS
!                  LOOPS
!                  KFFDBK
!                  CLDWTR

! ATTRIBUTES:
!   LANGUAGE: FORTRAN 90
!   MACHINE : IBM SP
!$$$
!----------------------------------------------------------------------
    INCLUDE "parmeta.f90"
    INCLUDE "mpp.h"
!----------------------------------------------------------------------
    PARAMETER(LP1=LM+1,JAM=6+2*(JM-10))
!----------------------------------------------------------------------
    INCLUDE "COMM_CTLBLK.f90"
    INCLUDE "COMM_VRBLS.f90"
    INCLUDE "COMM_DYNAM.f90"
    INCLUDE "COMM_PVRBLS.f90"
    INCLUDE "COMM_ACMCLH.f90"
    INCLUDE "COMM_CNVCLD.f90"
    INCLUDE "COMM_MASKS.f90"
    INCLUDE "COMM_LOOPS.f90"
    INCLUDE "COMM_KFFDBK.f90"
    INCLUDE "COMM_CLDWTR.f90"
    INCLUDE "COMM_ACMPRE.f90"
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------

!***  FEEDBACK CONVECTIVE TENDENCIES EACH TIME STEP

! omp parallel do
! omp& private(i,l,lg)
    DO 30 J=MYJS2,MYJE2
        DO 30 I=MYIS1,MYIE1
        
            IF(NCA(I,J) <= 0)GO TO 30
            LG=LMH(I,J)
        
            DO L=1,LG
                T(I,J,L)=T(I,J,L)+DTDT(I,J,L)*DT*HBM2(I,J)
                Q(I,J,L)=Q(I,J,L)+DQDT(I,J,L)*DT*HBM2(I,J)
                CWM(I,J,L)=CWM(I,J,L)+DQCDT(I,J,L)*DT*HBM2(I,J)
                TCUCN(I,J,L) = DTDT(I,J,L)*DT*HBM2(I,J)+ TCUCN(I,J,L)
            ENDDO
        
            CUPREC(I,J)=CUPREC(I,J)+RAINCV(I,J)*0.01
            ACPREC(I,J)=ACPREC(I,J)+RAINCV(I,J)*0.01
            CUPPT(I,J)=CUPPT(I,J)+RAINCV(I,J)*0.01
            PREC(I,J)=PREC(I,J)+RAINCV(I,J)*0.01
        
            PPTKF(I,J)=PPTKF(I,J)+RAINCV(I,J)*0.01
        
            IF(NCAD(I,J) > 0)THEN
                TNCA(I,J)=TNCA(I,J)+1.
                SPCLB(I,J)=SPCLB(I,J)+PCLB(I,J)
            
            !***  USE PEAK VALUE OF UMF INSTEAD OF 1 HOUR AVG.
            !***  DO THE SAME FOR PSRC.
            
                SPSRC(I,J)=AMAX1(PSRC(I,J),SPSRC(I,J))
                SUMFB(I,J)=AMAX1(UMFB(I,J),SUMFB(I,J))
                NCAD(I,J)=NCAD(I,J)-1
            ENDIF
        !----------------------------------------------------------------------
        
        !***  UPDATE THE CONVECTIVE TIME SCALE ARRAY.
        
            NCA(I,J)=NCA(I,J)-1
        
            IF(NCA(I,J) == 0)THEN
                RAINCV(I,J)=0.
            
                DO L=1,LM
                    DTDT(I,J,L)=0.
                    DQDT(I,J,L)=0.
                    DQCDT(I,J,L)=0.
                ENDDO
            
                PSRC(I,J) = 0.
                PCLB(I,J) = 0.
                UMFB(I,J) = 0.
            ENDIF
        !----------------------------------------------------------------------
    30 END DO
!----------------------------------------------------------------------
    RETURN
    END SUBROUTINE KFTEND
